//  Diffusion Client Library for iOS, OS X and tvOS
//
//  Copyright (c) 2016 Push Technology Ltd., All Rights Reserved.
//
//  Use is subject to license terms.
//
//  NOTICE: All information contained herein is, and remains the
//  property of Push Technology. The intellectual and technical
//  concepts contained herein are proprietary to Push Technology and
//  may be covered by U.S. and Foreign Patents, patents in process, and
//  are protected by trade secret or copyright law.

@import Foundation;
#import <Diffusion/PTDiffusionTopicType.h>

NS_ASSUME_NONNULL_BEGIN

/**
 @brief The specification of a topic.
 
 A topic is specified in terms of its topic type and a map of optional property
 settings which can alter the default behavior of the topic.

 @since 5.9
 */
@interface PTDiffusionTopicSpecification : NSObject

-(instancetype)init NS_UNAVAILABLE;

/**
 The type of topic.

 @since 5.9
 */
@property(nonatomic, readonly) PTDiffusionTopicType type;

/**
 The topic properties. May be empty.
 
 @since 5.9
 */
@property(nonatomic, readonly) NSDictionary<NSString *, NSString *>* properties;

/**
 Compares the receiver to the given topic specification.

 @param topicSpecification The topic specification object with which to compare the receiver.

 @return `YES` if the topic specification is equal to the receiver, otherwise `NO`.

 @since 5.9
 */
-(BOOL)isEqualToTopicSpecification:(nullable PTDiffusionTopicSpecification *)topicSpecification;

/**
 Key of boolean topic property indicating whether a topic should publish only
 values.

 By default, topics with a topic data type that supports deltas will publish a
 delta when doing so is more efficient than publishing the value.
 Setting this property to "true" disables this behavior so that deltas are never
 published. This results in more data being transmitted, but allows the client
 to only have to process values and not deltas.

 If this property is not set (or set to something other than "true") and a topic
 publishes deltas, clients need to use value streams to process the topics.
 If a client uses topic streams, it needs to process the deltas manually.

 If the client is using value streams to process the topics, it is not
 recommended to set this property as it will only lead to higher bandwidth usage.

 For topic data types that do not have delta support this value is irrelevant as
 only values would ever be published anyway.
 
 @since 5.9
 */
+(NSString *)publishValuesOnlyPropertyKey;

/**
 Key of boolean topic property indicating whether a topic should validate
 inbound values.

 By default, the server does not validate received values before sending them on
 to clients. Invalid or corrupt values will be stored in the topic and passed on
 to clients. If this property is set to "true", the server will perform
 additional validation on values to check that it is a valid instance of the
 data type, and if it is not then it will return an error to the updater and not
 update the topic.

 If this value is not set (or set to something other than "true"), no server
 validation of inbound values is performed. This is the recommended setting as
 there is a performance overhead to validation and a client using a value
 updater could not send invalid values anyway.
 
 @since 5.9
 */
+(NSString *)validateValuesPropertyKey;

/**
 Constant string representing the value to be used for boolean properties to
 represent "true".
 
 @since 5.9
 */
+(NSString *)truePropertyValue;

@end

NS_ASSUME_NONNULL_END
